{% extends "colorbox/popup_form_base.html" %}
{% load i18n %}{% load url from future %}
{% block title %}{% trans "Change the article template" %}{% endblock %}
{% block form_url %}{% url "coop_cms_change_template" article.id %}{% endblock %}
