{% extends "coop_cms/test_app/base.html" %}
{% load i18n coop_edition %}

{% block content %}
<ul>
{% for object in object_list %}
    <li>{{ object.id}} {{ object.other_field }}</li>
{% endfor %}
</ul>
{% endblock %}