{% extends "colorbox/popup_form_base.html" %}
{% load i18n %}{% load url from future %}
{% block title %}{{ title }}{% endblock %}
{% block form_url %}{% url "coop_cms_set_homepage" article.id %}{% endblock %}
