{% extends "registration/base.html" %}{% load i18n %}{% load url from future %}
{% block registration_content %}
<h2>{% trans "Password modification" %}</h2>

<form action="" method="post" class="registration-form">{% csrf_token %}
<table>
    {{form.as_table}}
<tr>
    <td>&nbsp;</td>
    <td><input type="submit" value="{% trans "Change password" %}" /></td>
</tr>
</table>
</form>
{% endblock %}