{% load coop_edition %}
{% if COOP_CMS_JQUERY_VERSION %}
    <script type="text/javascript" charset="utf-8" src="{{ STATIC_URL }}{{COOP_CMS_JQUERY_VERSION}}"></script>
{% else %}
    {% if debug %}
    <script type="text/javascript" charset="utf-8" src="{{ STATIC_URL }}js/jquery-1.11.3.js"></script>
    {% else %}
    <script type="text/javascript" charset="utf-8" src="{{ STATIC_URL }}js/jquery-1.11.3.min.js"></script>
    {% endif %}
{% endif %}
{% if_cms_edition %}
    <!--- <script type="text/javascript" charset="utf-8" src="{{ STATIC_URL }}js/jquery-migrate-1.2.1.min.js"></script>--->
    {% cms_form_media %}
{% else %}
    {% if request.user.is_authenticated %}
        <script type="text/javascript" charset="utf-8" src="{{ STATIC_URL }}js/jquery.colorbox-min.js?v=2"></script>
        <script type="text/javascript" charset="utf-8" src="{{ STATIC_URL }}js/jquery.form.js?v=2"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}js/colorbox.coop.js?v=2"></script>
        <link rel="stylesheet" href="{{ STATIC_URL }}css/colorbox.css?v=2" type="text/css" />
    {% endif %}
{% endif %}