{% extends "registration/base.html" %}{% load i18n %}{% load url from future %}
{% block registration_content %}
<h2>{% trans "The password has been changed" %}</h2>
<p>
    {% trans "Login" %} <a href="{% url "django.contrib.auth.views.login" %}">{% trans "Click Here" %}</a>
</p>
{% endblock %}