{% extends "registration/base.html" %}{% load i18n %}{% load url from future %}
{% block registration_content %}
<h2>{% trans "Password reset" %}</h2>
<form class="registration-form" action="" method="post">{% csrf_token %}
<table>
    {{form.as_table}}
    <tr>
    <td>&nbsp;</td>
    <td><input class="button" type="submit" value="{% trans "Reset" %}" /></td>
    <tr>    
</table>
<div class="registration-other-actions">
    <a href="{% url "django.contrib.auth.views.login" %}">{% trans "Login" %}</a>
</div>
</form>
{% endblock %}